@@ -101,6 +101,12 @@ STDCALL void WRAP_EXPORT(KeStallExecutionProcessor)
 	udelay(usecs);
 }
 
+STDCALL KIRQL WRAP_EXPORT(KeGetCurrentIrql)
+	(void)
+{
+	return current_irql();
+}
+
 _FASTCALL KIRQL WRAP_EXPORT(KfRaiseIrql)
 	(FASTCALL_DECL_1(KIRQL newirql))
 {
@@ -134,7 +140,6 @@ _FASTCALL void WRAP_EXPORT(KfReleaseSpinLock)
 {
 	TRACEENTER4("lock = %p, irql = %d", lock, newirql);
 	kspin_unlock_irql(lock, newirql);
-
 	TRACEEXIT4(return);
 }
 
@@ -144,7 +149,6 @@ _FASTCALL void WRAP_EXPORT(KefAcquireSpinLockAtDpcLevel)
 	KIRQL irql;
 
 	TRACEENTER4("lock = %p", lock);
-
 #if DEBUG_IRQL
 	irql = current_irql();
 	if (irql != DISPATCH_LEVEL)
@@ -163,7 +167,6 @@ _FASTCALL void WRAP_EXPORT(KefReleaseSpinLockFromDpcLevel)
 		ERROR("irql != DISPATCH_LEVEL");
 #endif
 	kspin_unlock(lock);
-
 	TRACEEXIT4(return);
 }
 
