@@ -12,6 +12,7 @@
  *  GNU General Public License for more details.
  *
  */
+
 #include <linux/types.h>
 #include <linux/module.h>
 #include <linux/delay.h>
@@ -24,49 +25,85 @@
 
 static struct list_head wrap_allocs;
 static KSPIN_LOCK wrap_allocs_lock;
+static struct nt_list wrapper_timer_list;
+static struct timer_list shared_data_timer;
+
+extern KSPIN_LOCK ntoskernel_lock;
+extern struct work_struct kdpc_work;
+
+struct kuser_shared_data kuser_shared_data;
+static void update_user_shared_data_proc(unsigned long data);
 
 int misc_funcs_init(void)
 {
 	INIT_LIST_HEAD(&wrap_allocs);
 	kspin_lock_init(&wrap_allocs_lock);
+	InitializeListHead(&wrapper_timer_list);
+
+	memset(&kuser_shared_data, 0, sizeof(kuser_shared_data));
+	init_timer(&shared_data_timer);
+	shared_data_timer.function = &update_user_shared_data_proc;
+#if defined(CONFIG_X86_64) && defined(INPROCOMM_AMD64)
+	*((ULONG64 *)SHARED_SYSTEM_TIME) = ticks_1601();
+	shared_data_timer.data = (unsigned long)0;
+	shared_data_timer.expires = jiffies + 10 * HZ / 1000;
+	add_timer(&shared_data_timer);
+#endif
 	return 0;
 }
 
 /* called when a handle is being removed */
-void misc_funcs_exit_handle(struct ndis_handle *handle)
+void misc_funcs_exit_device(struct wrapper_dev *wd)
 {
-	char canceled;
+	BOOLEAN canceled;
 	KIRQL irql;
 
-	/* cancel any timers left by bugyy windows driver
-	 * Also free the memory for timers
-	 */
+	/* cancel any timers left by bugyy windows driver Also free
+	 * the memory for timers */
 	while (1) {
-		struct wrapper_timer *timer;
+		struct nt_list *ent;
+		struct wrapper_timer *wrapper_timer;
 
-		irql = kspin_lock_irql(&handle->timers_lock, DISPATCH_LEVEL);
-		if (list_empty(&handle->timers)) {
-			kspin_unlock_irql(&handle->timers_lock, irql);
+		irql = kspin_lock_irql(&wd->timer_lock, DISPATCH_LEVEL);
+		ent = RemoveHeadList(&wd->wrapper_timer_list);
+		kspin_unlock_irql(&wd->timer_lock, irql);
+		if (!ent)
 			break;
-		}
-
-		timer = list_entry(handle->timers.next,
-				   struct wrapper_timer, list);
-		list_del(&timer->list);
-		kspin_unlock_irql(&handle->timers_lock, irql);
-
-		DBGTRACE1("fixing up timer %p, timer->list %p",
-			  timer, &timer->list);
-		wrapper_cancel_timer(timer, &canceled);
-		wrap_kfree(timer);
+		wrapper_timer = container_of(ent, struct wrapper_timer, list);
+		wrapper_cancel_timer(wrapper_timer, &canceled);
+		if (canceled == TRUE)
+			WARNING("Buggy Windows driver '%s' left a timer (%p)"
+				"running - fixed it", wd->driver->name,
+				wrapper_timer->ktimer);
+		wrap_kfree(wrapper_timer);
 	}
 }
 
 /* called when module is being removed */
 void misc_funcs_exit(void)
 {
+	KIRQL irql;
+
+	/* free kernel (Ke) timers */
+	while (1) {
+		struct nt_list *ent;
+		struct wrapper_timer *wrapper_timer;
+		BOOLEAN canceled;
+
+		irql = kspin_lock_irql(&ntoskernel_lock, DISPATCH_LEVEL);
+		ent = RemoveTailList(&wrapper_timer_list);
+		kspin_unlock_irql(&ntoskernel_lock, irql);
+		if (!ent)
+			break;
+		wrapper_timer = container_of(ent, struct wrapper_timer, list);
+		wrapper_cancel_timer(wrapper_timer, &canceled);
+		if (canceled == TRUE)
+			WARNING("Buggy Windows driver left a timer (%p)"
+				"running - fixed it", wrapper_timer);
+		wrap_kfree(wrapper_timer);
+	}
 	/* free all pointers on the allocated list */
-	kspin_lock(&wrap_allocs_lock);
+	irql = kspin_lock_irql(&wrap_allocs_lock, DISPATCH_LEVEL);
 	while (!list_empty(&wrap_allocs)) {
 		struct wrap_alloc *alloc;
 
@@ -75,11 +112,28 @@ void misc_funcs_exit(void)
 		kfree(alloc->ptr);
 		kfree(alloc);
 	}
-	kspin_unlock(&wrap_allocs_lock);
+	kspin_unlock_irql(&wrap_allocs_lock, irql);
+
+#if defined(CONFIG_X86_64) && defined(INPROCOMM_AMD64)
+	del_timer_sync(&shared_data_timer);
+#endif
 
 	TRACEEXIT4(return);
 }
 
+static void update_user_shared_data_proc(unsigned long data)
+{
+#if defined(CONFIG_X86_64) && defined(INPROCOMM_AMD64)
+	/* timer is scheduled every 10ms and the system timer is in
+	 * 100ns */
+	*((ULONG64 *)SHARED_SYSTEM_TIME) = ticks_1601();
+	*((ULONG64 *)SHARED_INTERRUPT_TIME) = jiffies * TICKSPERSEC / HZ;
+	*((ULONG64 *)SHARED_TICK_COUNT) = jiffies;
+#endif
+	shared_data_timer.expires = jiffies + 10 * HZ / 1000;
+	add_timer(&shared_data_timer);
+}
+
 /* allocate memory with given flags and add it to list of allocated pointers;
  * if a driver doesn't free this memory for any reason (buggy driver or we
  * allocate space behind driver's back since we need more space than
@@ -89,6 +143,7 @@ void misc_funcs_exit(void)
 void *wrap_kmalloc(size_t size, int flags)
 {
 	struct wrap_alloc *alloc;
+	KIRQL irql;
 
 	TRACEENTER4("size = %lu, flags = %d", (unsigned long)size, flags);
 
@@ -103,9 +158,9 @@ void *wrap_kmalloc(size_t size, int flags)
 		kfree(alloc);
 		return NULL;
 	}
-	kspin_lock(&wrap_allocs_lock);
+	irql = kspin_lock_irql(&wrap_allocs_lock, DISPATCH_LEVEL);
 	list_add(&alloc->list, &wrap_allocs);
-	kspin_unlock(&wrap_allocs_lock);
+	kspin_unlock_irql(&wrap_allocs_lock, irql);
 	DBGTRACE4("%p, %p", alloc, alloc->ptr);
 	TRACEEXIT4(return alloc->ptr);
 }
@@ -114,9 +169,10 @@ void *wrap_kmalloc(size_t size, int flags)
 void wrap_kfree(void *ptr)
 {
 	struct list_head *cur, *tmp;
+	KIRQL irql;
 
 	TRACEENTER4("%p", ptr);
-	kspin_lock(&wrap_allocs_lock);
+	irql = kspin_lock_irql(&wrap_allocs_lock, DISPATCH_LEVEL);
 	list_for_each_safe(cur, tmp, &wrap_allocs) {
 		struct wrap_alloc *alloc;
 
@@ -128,141 +184,157 @@ void wrap_kfree(void *ptr)
 			break;
 		}
 	}
-	kspin_unlock(&wrap_allocs_lock);
+	kspin_unlock_irql(&wrap_allocs_lock, irql);
 	TRACEEXIT4(return);
 }
 
 void wrapper_timer_handler(unsigned long data)
 {
-	struct wrapper_timer *timer = (struct wrapper_timer *)data;
+	struct ktimer *ktimer = (struct ktimer *)data;
+	struct wrapper_timer *wrapper_timer;
 	struct kdpc *kdpc;
-	STDCALL void (*miniport_timer)(void *specific1, void *ctx,
-				       void *specific2, void *specific3);
 
-	KIRQL irql;
+	TRACEENTER5("%p", ktimer);
+	wrapper_timer = ktimer->wrapper_timer;
 
-	TRACEENTER5("%p", timer);
 #ifdef DEBUG_TIMER
-	BUG_ON(timer == NULL);
-	BUG_ON(timer->wrapper_timer_magic != WRAPPER_TIMER_MAGIC);
-	BUG_ON(timer->kdpc == NULL);
+	BUG_ON(wrapper_timer == NULL);
+	BUG_ON(wrapper_timer->wrapper_timer_magic != WRAPPER_TIMER_MAGIC);
+	BUG_ON(wrapper_timer->kdpc == NULL);
 #endif
 
-	/* don't add the timer if aperiodic; see wrapper_cancel_timer
-	 * protect access to kdpc, repeat, and active via spinlock */
-	irql = kspin_lock_irql(&timer->lock, DISPATCH_LEVEL);
-	kdpc = timer->kdpc;
-	if (timer->repeat) {
-		timer->timer.expires = jiffies + timer->repeat;
-		add_timer(&timer->timer);
-	} else
-		timer->active = 0;
-	kspin_unlock_irql(&timer->lock, irql);
-
-	miniport_timer = kdpc->func;
-
-	/* call the handler after restarting in case it cancels itself */
-	if (miniport_timer) {
-		KIRQL irql;
-		irql = raise_irql(DISPATCH_LEVEL);
-		LIN2WIN4(miniport_timer, kdpc, kdpc->ctx, kdpc->arg1,
-			 kdpc->arg2);
-		lower_irql(irql);
+	kdpc = wrapper_timer->kdpc;
+	if (kdpc) {
+		kspin_lock(&ntoskernel_lock);
+		insert_kdpc_work(kdpc);
+		kspin_unlock(&ntoskernel_lock);
 	}
+	kspin_lock(&wrapper_timer->lock);
+	/* don't add the timer if aperiodic - see wrapper_cancel_timer;
+	 * protect access to kdpc, repeat, and active via spinlock */
+	if (wrapper_timer->repeat) {
+		wrapper_timer->timer.expires = jiffies + wrapper_timer->repeat;
+		add_timer(&wrapper_timer->timer);
+	} else
+		wrapper_timer->active = 0;
+	kspin_unlock(&wrapper_timer->lock);
+
+	KeSetEvent((struct kevent *)ktimer, 0, FALSE);
 
 	TRACEEXIT5(return);
 }
 
-void wrapper_init_timer(struct ktimer *ktimer, void *handle)
+/* we don't initialize ktimer event's signal here; that is caller's
+ * responsibility */
+
+/* NDIS (miniport) timers set kdpc during timer initialization and Ke
+ * timers set it during timer setup, so it is passed to both
+ * wrapper_init_timer and wrapper_set_timer */
+void wrapper_init_timer(struct ktimer *ktimer, void *handle, struct kdpc *kdpc)
 {
 	struct wrapper_timer *wrapper_timer;
-	struct ndis_handle *ndis_handle = (struct ndis_handle *)handle;
+	struct wrapper_dev *wd = (struct wrapper_dev *)handle;
+	KIRQL irql;
 
 	TRACEENTER5("%s", "");
-	wrapper_timer = wrap_kmalloc(sizeof(struct wrapper_timer), GFP_ATOMIC);
+	/* we allocate memory for wrapper_timer behind driver's back
+	 * and there is no NDIS/DDK function where this memory can be
+	 * freed, so we use wrap_kmalloc so it gets freed when driver
+	 * is unloaded */
+	wrapper_timer = wrap_kmalloc(sizeof(*wrapper_timer), GFP_ATOMIC);
 	if (!wrapper_timer) {
 		ERROR("couldn't allocate memory for timer");
 		return;
 	}
 
-	memset(wrapper_timer, 27, sizeof(*wrapper_timer));
+	memset(wrapper_timer, 0, sizeof(*wrapper_timer));
 	init_timer(&wrapper_timer->timer);
-	wrapper_timer->timer.data = (unsigned long)wrapper_timer;
+	wrapper_timer->timer.data = (unsigned long)ktimer;
 	wrapper_timer->timer.function = &wrapper_timer_handler;
-	wrapper_timer->active = 0;
-	wrapper_timer->repeat = 0;
-	wrapper_timer->kdpc = NULL;
 #ifdef DEBUG_TIMER
 	wrapper_timer->wrapper_timer_magic = WRAPPER_TIMER_MAGIC;
 #endif
+	wrapper_timer->kdpc = kdpc;
+	wrapper_timer->ktimer = ktimer;
 	ktimer->wrapper_timer = wrapper_timer;
 	kspin_lock_init(&wrapper_timer->lock);
-	if (handle) {
-		kspin_lock(&ndis_handle->timers_lock);
-		list_add(&wrapper_timer->list, &ndis_handle->timers);
-		kspin_unlock(&ndis_handle->timers_lock);
+	if (wd) {
+		irql = kspin_lock_irql(&wd->timer_lock, DISPATCH_LEVEL);
+		InsertHeadList(&wd->wrapper_timer_list,
+			       &wrapper_timer->list);
+		kspin_unlock_irql(&wd->timer_lock, irql);
+	} else {
+		irql = kspin_lock_irql(&ntoskernel_lock, DISPATCH_LEVEL);
+		InsertHeadList(&wrapper_timer_list, &wrapper_timer->list);
+		kspin_unlock_irql(&ntoskernel_lock, irql);
 	}
 
-	DBGTRACE4("added timer %p, wrapper_timer->list %p\n",
-		  wrapper_timer, &wrapper_timer->list);
+	DBGTRACE4("added timer %p", wrapper_timer);
 	TRACEEXIT5(return);
 }
 
-int wrapper_set_timer(struct wrapper_timer *timer, unsigned long expires,
-		      unsigned long repeat, struct kdpc *kdpc)
+/* 'expires' is relative to jiffies, so when setting timer, add
+ * jiffies to it */
+int wrapper_set_timer(struct wrapper_timer *wrapper_timer,
+		      unsigned long expires, unsigned long repeat,
+		      struct kdpc *kdpc)
 {
 	KIRQL irql;
+	struct ktimer *ktimer = wrapper_timer->ktimer;
 
-	TRACEENTER5("%p", timer);
-	if (!timer) {
+	TRACEENTER5("%p", wrapper_timer);
+	if (!wrapper_timer) {
 		ERROR("invalid timer");
 		return FALSE;
 	}
 
 #ifdef DEBUG_TIMER
-	if (timer->wrapper_timer_magic != WRAPPER_TIMER_MAGIC) {
+	if (wrapper_timer->wrapper_timer_magic != WRAPPER_TIMER_MAGIC) {
 		WARNING("timer %p is not initialized (%lu)",
-			timer, timer->wrapper_timer_magic);
-		timer->wrapper_timer_magic = WRAPPER_TIMER_MAGIC;
+			wrapper_timer, wrapper_timer->wrapper_timer_magic);
+		wrapper_timer->wrapper_timer_magic = WRAPPER_TIMER_MAGIC;
 	}
 #endif
 
 	/* timer handler also uses timer->repeat, active, and kdpc, so
 	 * protect in case of SMP */
-	irql = kspin_lock_irql(&timer->lock, DISPATCH_LEVEL);
+	KeClearEvent((struct kevent *)ktimer);
+	irql = kspin_lock_irql(&wrapper_timer->lock, DISPATCH_LEVEL);
 	if (kdpc)
-		timer->kdpc = kdpc;
-	timer->repeat = repeat;
-	if (timer->active) {
+		wrapper_timer->kdpc = kdpc;
+	wrapper_timer->repeat = repeat;
+	if (wrapper_timer->active) {
 		DBGTRACE4("modifying timer %p to %lu, %lu",
-			  timer, expires, repeat);
-		mod_timer(&timer->timer, expires);
-		kspin_unlock_irql(&timer->lock, irql);
+			  wrapper_timer, expires, repeat);
+		mod_timer(&wrapper_timer->timer, jiffies + expires);
+		kspin_unlock_irql(&wrapper_timer->lock, irql);
 		TRACEEXIT5(return TRUE);
 	} else {
 		DBGTRACE4("setting timer %p to %lu, %lu",
-			  timer, expires, repeat);
-		timer->timer.expires = expires;
-		timer->active = 1;
-		add_timer(&timer->timer);
-		kspin_unlock_irql(&timer->lock, irql);
+			  wrapper_timer, expires, repeat);
+		wrapper_timer->timer.expires = jiffies + expires;
+		wrapper_timer->active = 1;
+		add_timer(&wrapper_timer->timer);
+		kspin_unlock_irql(&wrapper_timer->lock, irql);
 		TRACEEXIT5(return FALSE);
 	}
 }
 
-void wrapper_cancel_timer(struct wrapper_timer *timer, char *canceled)
+void wrapper_cancel_timer(struct wrapper_timer *wrapper_timer,
+			  BOOLEAN *canceled)
 {
 	KIRQL irql;
+	struct ktimer *ktimer = wrapper_timer->ktimer;
 
-	TRACEENTER4("timer = %p, canceled = %p", timer, canceled);
-	if (!timer) {
-		ERROR("%s", "invalid timer");
+	TRACEENTER4("timer = %p, canceled = %p", wrapper_timer, canceled);
+	if (!wrapper_timer) {
+		ERROR("%s", "invalid wrapper_timer");
 		return;
 	}
 
 #ifdef DEBUG_TIMER
-	DBGTRACE4("canceling timer %p", timer);
-	BUG_ON(timer->wrapper_timer_magic != WRAPPER_TIMER_MAGIC);
+	DBGTRACE4("canceling timer %p", wrapper_timer);
+	BUG_ON(wrapper_timer->wrapper_timer_magic != WRAPPER_TIMER_MAGIC);
 #endif
 	/* timer handler also uses timer->repeat, so protect it in
 	 * case of SMP */
@@ -271,17 +343,29 @@ void wrapper_cancel_timer(struct wrapper_timer *timer, char *canceled)
 	 * tells the driver if the timer was deleted or not) here; nor
 	 * is del_timer_sync correct, as this function may be called
 	 * at DISPATCH_LEVEL */
-	irql = kspin_lock_irql(&timer->lock, DISPATCH_LEVEL);
-	if (timer->repeat) {
-		/* first mark as aperiodic, so timer function doesn't call
-		 * add_timer after del_timer returned */
-		timer->repeat = 0;
-		del_timer(&timer->timer);
-		/* periodic timers always return TRUE */
-		*canceled = TRUE;
+	irql = raise_irql(DISPATCH_LEVEL);
+	kspin_lock(&ntoskernel_lock);
+	remove_kdpc_work(wrapper_timer->kdpc);
+	kspin_unlock(&ntoskernel_lock);
+	kspin_lock(&wrapper_timer->lock);
+	if (wrapper_timer->active) {
+		if (wrapper_timer->repeat) {
+			/* first mark as aperiodic, so timer function
+			 * doesn't call add_timer after del_timer
+			 * returned */
+			wrapper_timer->repeat = 0;
+			del_timer(&wrapper_timer->timer);
+			/* periodic timers always return TRUE */
+			*canceled = TRUE;
+		} else
+			*canceled = del_timer(&wrapper_timer->timer);
+		wrapper_timer->active = 0;
 	} else
-		*canceled = del_timer(&timer->timer);
-	kspin_unlock_irql(&timer->lock, irql);
+		*canceled = FALSE;
+	kspin_unlock(&wrapper_timer->lock);
+	lower_irql(irql);
+	if (*canceled == TRUE)
+		KeClearEvent((struct kevent *)ktimer);
 	TRACEEXIT5(return);
 }
 
@@ -362,13 +446,58 @@ NOREGPARM INT WRAP_EXPORT(_win_strcmp)
 	return strcmp(s1, s2);
 }
 
-int stricmp(const char *s1, const char *s2)
+NOREGPARM INT WRAP_EXPORT(_win_stricmp)
+	(const char *s1, const char *s2)
 {
-	while (*s1 && *s2 && tolower(*s1) == tolower(*s2)) {
+	return stricmp(s1, s2);
+}
+
+NOREGPARM char *WRAP_EXPORT(_win_strncat)
+	(char *dest, const char *src, SIZE_T n)
+{
+	return strncat(dest, src, n);
+}
+
+NOREGPARM INT WRAP_EXPORT(_win_wcscmp)
+	(const wchar_t *s1, const wchar_t *s2)
+{
+	while (*s1 && *s2 && *s1 == *s2) {
 		s1++;
 		s2++;
 	}
-	return (int)*s1 - (int)*s2;
+	return *s1 - *s2;
+}
+
+NOREGPARM INT WRAP_EXPORT(_win_wcsicmp)
+	(const wchar_t *s1, const wchar_t *s2)
+{
+	while (*s1 && *s2 && tolower((char)*s1) == tolower((char)*s2)) {
+		s1++;
+		s2++;
+	}
+	return *s1 - *s2;
+}
+
+NOREGPARM SIZE_T WRAP_EXPORT(_win_wcslen)
+	(const wchar_t *s)
+{
+	SIZE_T i = 0;
+	while (s[i])
+		i++;
+	return i;
+}
+
+NOREGPARM wchar_t *WRAP_EXPORT(_win_wcsncpy)
+	(wchar_t *dest, const wchar_t *src, SIZE_T n)
+{
+	SIZE_T i = 0;
+	while (i < n && src[i]) {
+		dest[i] = src[i];
+		i++;
+	}
+	if (i < n)
+		dest[i] = 0;
+	return dest;
 }
 
 NOREGPARM INT WRAP_EXPORT(_win_tolower)
@@ -383,12 +512,6 @@ NOREGPARM INT WRAP_EXPORT(_win_toupper)
 	return toupper(c);
 }
 
-NOREGPARM void *WRAP_EXPORT(_win_memcpy)
-	(void *to, const void *from, SIZE_T n)
-{
-	return memcpy(to, from, n);
-}
-
 NOREGPARM void *WRAP_EXPORT(_win_strcpy)
 	(void *to, const void *from)
 {
@@ -407,12 +530,6 @@ NOREGPARM char *WRAP_EXPORT(_win_strchr)
 	return strchr(s, c);
 }
 
-NOREGPARM void *WRAP_EXPORT(_win_memset)
-	(void *s, char c, SIZE_T count)
-{
-	return memset(s, c, count);
-}
-
 NOREGPARM void *WRAP_EXPORT(_win_memmove)
 	(void *to, void *from, SIZE_T count)
 {
@@ -425,6 +542,19 @@ NOREGPARM void *WRAP_EXPORT(_win_memchr)
 	return memchr(s, c, n);
 }
 
+/* memcpy and memset are macros so we can't map them */
+NOREGPARM void *WRAP_EXPORT(_win_memcpy)
+	(void *to, const void *from, SIZE_T n)
+{
+	return memcpy(to, from, n);
+}
+
+NOREGPARM void *WRAP_EXPORT(_win_memset)
+	(void *s, char c, SIZE_T count)
+{
+	return memset(s, c, count);
+}
+
 NOREGPARM void WRAP_EXPORT(_win_srand)
 	(UINT seed)
 {
@@ -529,6 +659,18 @@ STDCALL void WRAP_EXPORT(RtlCopyMemory)
 	memcpy(dst, src, length);
 }
 
+STDCALL void WRAP_EXPORT(RtlZeroMemory)
+	(void *dst, SIZE_T length)
+{
+	memset(dst, 0, length);
+}
+
+STDCALL void WRAP_EXPORT(RtlSecureZeroMemory)
+	(void *dst, SIZE_T length)
+{
+	memset(dst, 0, length);
+}
+
 STDCALL LONG WRAP_EXPORT(RtlCompareString)
 	(const struct ansi_string *s1, const struct ansi_string *s2,
 	 BOOLEAN case_insensitive)
@@ -607,11 +749,38 @@ STDCALL void WRAP_EXPORT(RtlCopyUnicodeString)
 		/* append terminating '\0' if enough space */
 		if (len < dst->buflen)
 			dst->buf[len] = 0;
-	}
-	else dst->len = 0;
+	} else
+		dst->len = 0;
 	TRACEEXIT1(return);
 }
 
+STDCALL NTSTATUS WRAP_EXPORT(RtlAppendUnicodeToString)
+	(struct unicode_string *dst, wchar_t *src)
+{
+	int i;
+
+	for (i = 0; src[i] != 0 && dst->len + i < dst->buflen; i++)
+		dst->buf[dst->len + i] = src[i];
+	if (src[i] != 0)
+		return STATUS_BUFFER_TOO_SMALL;
+	dst->len += i;
+	return STATUS_SUCCESS;
+}
+
+STDCALL NTSTATUS WRAP_EXPORT(RtlAppendUnicodeStringToString)
+	(struct unicode_string *dst, struct unicode_string *src)
+{
+	int i;
+
+	if (dst->buflen < dst->len + src->len)
+		return STATUS_BUFFER_TOO_SMALL;
+
+	for (i = 0; i < src->len; i++)
+		dst->buf[dst->len + i] = src->buf[i];
+	dst->len += i;
+	return STATUS_SUCCESS;
+}
+
 STDCALL NTSTATUS WRAP_EXPORT(RtlAnsiStringToUnicodeString)
 	(struct unicode_string *dst, struct ansi_string *src, BOOLEAN dup)
 {
@@ -624,12 +793,12 @@ STDCALL NTSTATUS WRAP_EXPORT(RtlAnsiStringToUnicodeString)
 		wchar_t *buf = kmalloc((src->buflen+1) * sizeof(wchar_t),
 				       GFP_KERNEL);
 		if (!buf)
-			TRACEEXIT1(return NDIS_STATUS_FAILURE);
+			TRACEEXIT1(return STATUS_FAILURE);
 		dst->buf = buf;
 		dst->buflen = (src->buflen+1) * sizeof(wchar_t);
 	}
 	else if (dst->buflen < (src->len+1) * sizeof(wchar_t))
-		TRACEEXIT1(return NDIS_STATUS_FAILURE);
+		TRACEEXIT1(return STATUS_FAILURE);
 
 	dst->len = src->len * sizeof(wchar_t);
 	d = dst->buf;
@@ -640,7 +809,7 @@ STDCALL NTSTATUS WRAP_EXPORT(RtlAnsiStringToUnicodeString)
 	d[i] = 0;
 
 	DBGTRACE2("len = %d", dst->len);
-	TRACEEXIT2(return NDIS_STATUS_SUCCESS);
+	TRACEEXIT2(return STATUS_SUCCESS);
 }
 
 STDCALL NTSTATUS WRAP_EXPORT(RtlUnicodeStringToAnsiString)
@@ -657,11 +826,11 @@ STDCALL NTSTATUS WRAP_EXPORT(RtlUnicodeStringToAnsiString)
 		char *buf = kmalloc((src->buflen+1) / sizeof(wchar_t),
 				    GFP_KERNEL);
 		if (!buf)
-			return NDIS_STATUS_FAILURE;
+			return STATUS_FAILURE;
 		dst->buf = buf;
 		dst->buflen = (src->buflen+1) / sizeof(wchar_t);
 	} else if (dst->buflen < (src->len+1) / sizeof(wchar_t))
-		return NDIS_STATUS_FAILURE;
+		return STATUS_FAILURE;
 
 	dst->len = src->len / sizeof(wchar_t);
 	s = src->buf;
@@ -672,7 +841,7 @@ STDCALL NTSTATUS WRAP_EXPORT(RtlUnicodeStringToAnsiString)
 
 	DBGTRACE2("len = %d", dst->len);
 	DBGTRACE2("string: %s", dst->buf);
-	TRACEEXIT2(return NDIS_STATUS_SUCCESS);
+	TRACEEXIT2(return STATUS_SUCCESS);
 }
 
 STDCALL NTSTATUS WRAP_EXPORT(RtlUnicodeStringToInteger)
@@ -751,7 +920,7 @@ STDCALL NTSTATUS WRAP_EXPORT(RtlIntegerToUnicodeString)
 	if (base == 0)
 		base = 10;
 	if (!(base == 2 || base == 8 || base == 10 || base == 16))
-		return NDIS_STATUS_INVALID_PARAMETER;
+		return STATUS_INVALID_PARAMETER;
 	for (i = 0; value && i < sizeof(string); i++) {
 		int r;
 		r = value % base;
@@ -765,7 +934,7 @@ STDCALL NTSTATUS WRAP_EXPORT(RtlIntegerToUnicodeString)
 	if (i < sizeof(string))
 		string[i] = 0;
 	else
-		return NDIS_STATUS_BUFFER_TOO_SHORT;
+		return STATUS_BUFFER_TOO_SMALL;
 
 	ansi.buf = string;
 	ansi.len = strlen(string);
@@ -774,13 +943,10 @@ STDCALL NTSTATUS WRAP_EXPORT(RtlIntegerToUnicodeString)
 }
 
 STDCALL void WRAP_EXPORT(RtlInitUnicodeString)
-	(struct unicode_string *dest, wchar_t *src)
+	(struct unicode_string *dest, const wchar_t *src)
 {
-	struct unicode_string *uc;
-
 	TRACEENTER1("%s", "");
-	uc = dest;
-	if (uc == NULL)
+	if (dest == NULL)
 		TRACEEXIT1(return);
 	if (src == NULL) {
 		dest->len = dest->buflen = 0;
@@ -789,7 +955,7 @@ STDCALL void WRAP_EXPORT(RtlInitUnicodeString)
 		int i = 0;
 		while (src[i])
 			i++;
-		dest->buf = src;
+		dest->buf = (wchar_t *)src;
 		dest->len = dest->buflen = i * 2;
 	}
 	TRACEEXIT1(return);
@@ -859,6 +1025,12 @@ STDCALL NTSTATUS WRAP_EXPORT(RtlWriteRegistryValue)
 	TRACEEXIT5(return STATUS_SUCCESS);
 }
 
+STDCALL NTSTATUS WRAP_EXPORT(RtlDeleteRegistryValue)
+	(ULONG relative, wchar_t *path, wchar_t *name)
+{
+	return STATUS_SUCCESS;
+}
+
 STDCALL void WRAP_EXPORT(RtlAssert)
 	(char *failed_assertion, char *file_name, ULONG line_num,
 	 char *message)
@@ -886,13 +1058,22 @@ ULONGLONG ticks_1601(void)
 	ULONGLONG ticks;
 
 	do_gettimeofday(&now);
-	ticks = (ULONGLONG) now.tv_sec * TICKSPERSEC;
+	ticks = (ULONGLONG)now.tv_sec * TICKSPERSEC;
 	ticks += now.tv_usec * 10 + TICKS_1601_TO_1970;
 	return ticks;
 }
 
 void WRAP_EXPORT(RtlUnwind)(void){UNIMPL();}
 
+int stricmp(const char *s1, const char *s2)
+{
+	while (*s1 && *s2 && tolower(*s1) == tolower(*s2)) {
+		s1++;
+		s2++;
+	}
+	return (int)*s1 - (int)*s2;
+}
+
 void *get_sp(void)
 {
 	ULONG_PTR i;
