@@ -72,16 +72,10 @@ struct ndis_buffer_pool {
 #define fPACKET_ALLOCATED_BY_NDIS		0x80
 
 enum ndis_per_packet_info {
-	TcpIpChecksumPacketInfo,
-	IpSecPacketInfo,
-	TcpLargeSendPacketInfo,
-	ClassificationHandlePacketInfo,
-	NdisReserved,
-	ScatterGatherListPacketInfo,
-	Ieee8021QInfo,
-	OriginalPacketInfo,
-	PacketCancelId,
-	MaxPerPacketInfo
+	TcpIpChecksumPacketInfo, IpSecPacketInfo, TcpLargeSendPacketInfo,
+	ClassificationHandlePacketInfo, NdisReserved,
+	ScatterGatherListPacketInfo, Ieee8021QInfo, OriginalPacketInfo,
+	PacketCancelId, MaxPerPacketInfo
 };
 
 struct ndis_packet_extension {
@@ -158,6 +152,11 @@ struct ndis_packet {
 	UINT look_ahead_size;
 };
 
+struct wrap_ndis_packet {
+	struct nt_list list;
+	struct ndis_packet ndis_packet;
+};
+
 struct ndis_packet_pool {
 	int max_descr;
 	int num_allocated_descr;
@@ -166,27 +165,17 @@ struct ndis_packet_pool {
 };
 
 enum ndis_device_pnp_event {
-	NdisDevicePnPEventQueryRemoved,
-	NdisDevicePnPEventRemoved,
-	NdisDevicePnPEventSurpriseRemoved,
-	NdisDevicePnPEventQueryStopped,
-	NdisDevicePnPEventStopped,
-	NdisDevicePnPEventPowerProfileChanged,
+	NdisDevicePnPEventQueryRemoved, NdisDevicePnPEventRemoved,
+	NdisDevicePnPEventSurpriseRemoved, NdisDevicePnPEventQueryStopped,
+	NdisDevicePnPEventStopped, NdisDevicePnPEventPowerProfileChanged,
 	NdisDevicePnPEventMaximum
 };
 
 enum ndis_request_type {
-	NdisRequestQueryInformation,
-	NdisRequestSetInformation,
-	NdisRequestQueryStatistics,
-	NdisRequestOpen,
-	NdisRequestClose,
-	NdisRequestSend,
-	NdisRequestTransferData,
-	NdisRequestReset,
-	NdisRequestGeneric1,
-	NdisRequestGeneric2,
-	NdisRequestGeneric3,
+	NdisRequestQueryInformation, NdisRequestSetInformation,
+	NdisRequestQueryStatistics, NdisRequestOpen, NdisRequestClose,
+	NdisRequestSend, NdisRequestTransferData, NdisRequestReset,
+	NdisRequestGeneric1, NdisRequestGeneric2, NdisRequestGeneric3,
 	NdisRequestGeneric4
 };
 
@@ -212,43 +201,25 @@ struct ndis_request {
 };
 
 enum ndis_medium {
-	NdisMedium802_3,
-	NdisMedium802_5,
-	NdisMediumFddi,
-	NdisMediumWan,
-	NdisMediumLocalTalk,
-	NdisMediumDix,
-	NdisMediumArcnetRaw,
-	NdisMediumArcnet878_2,
-	NdisMediumAtm,
-	NdisMediumWirelessWan,
-	NdisMediumIrda,
-	NdisMediumBpc,
-	NdisMediumCoWan,
-	NdisMedium1394,
-	NdisMediumMax
+	NdisMedium802_3, NdisMedium802_5, NdisMediumFddi, NdisMediumWan,
+	NdisMediumLocalTalk, NdisMediumDix, NdisMediumArcnetRaw,
+	NdisMediumArcnet878_2, NdisMediumAtm, NdisMediumWirelessWan,
+	NdisMediumIrda, NdisMediumBpc, NdisMediumCoWan,
+	NdisMedium1394, NdisMediumMax
 };
 
 enum ndis_phys_medium {
-	NdisPhysicalMediumUnspecified,
-	NdisPhysicalMediumWirelessLan,
-	NdisPhysicalMediumCableModem,
-	NdisPhysicalMediumPhoneLine,
-	NdisPhysicalMediumPowerLine,
-	NdisPhysicalMediumDSL,
-	NdisPhysicalMediumFibreChannel,
-	NdisPhysicalMedium1394,
-	NdisPhysicalMediumWirelessWan,
-	NdisPhysicalMediumMax
+	NdisPhysicalMediumUnspecified, NdisPhysicalMediumWirelessLan,
+	NdisPhysicalMediumCableModem, NdisPhysicalMediumPhoneLine,
+	NdisPhysicalMediumPowerLine, NdisPhysicalMediumDSL,
+	NdisPhysicalMediumFibreChannel, NdisPhysicalMedium1394,
+	NdisPhysicalMediumWirelessWan, NdisPhysicalMediumMax
 };
 
 enum ndis_pm_state {
 	NdisDeviceStateUnspecified = 0,
-	NdisDeviceStateD0,
-	NdisDeviceStateD1,
-	NdisDeviceStateD2,
-	NdisDeviceStateD3,
-	NdisDeviceStateMaximum
+	NdisDeviceStateD0, NdisDeviceStateD1, NdisDeviceStateD2,
+	NdisDeviceStateD3, NdisDeviceStateMaximum
 };
 
 typedef void (*ndis_isr_handler)(unsigned int *taken, unsigned int *callme,
@@ -275,7 +246,7 @@ struct miniport_char {
 	/* Start miniport driver */
 	UINT (*init)(UINT *status, UINT *medium_index,
 		     enum ndis_medium medium[], UINT medium_array_size,
-		     void *ndis_handle, void *conf_handle) STDCALL;
+		     void *handle, void *conf_handle) STDCALL;
 
 	/* Interrupt TH */
 	ndis_isr_handler isr;
@@ -354,15 +325,15 @@ struct ndis_rw_lock {
     union ndis_rw_lock_refcount ref_count[MAXIMUM_PROCESSORS];
 };
 
-struct handle_ctx_entry {
+struct wd_ctx_entry {
 	struct list_head list;
-	void *handle;
+	void *wd;
 	void *ctx;
 };
 
 struct ndis_sched_work_item {
 	void *ctx;
-	void (*func)(struct ndis_sched_work_item *, void *) STDCALL;
+	void (*func)(struct ndis_sched_work_item *, void *ctx) STDCALL;
 	UCHAR reserved[8 * sizeof(void *)];
 };
 
@@ -385,17 +356,15 @@ struct ndis_free_mem_work_item {
 };
 
 enum ndis_work_entry_type {
-	NDIS_SCHED_WORK_ITEM,
-	NDIS_ALLOC_MEM_WORK_ITEM,
-	NDIS_FREE_MEM_WORK_ITEM,
-	NDIS_IO_WORK_ITEM,
+	NDIS_SCHED_WORK_ITEM, NDIS_ALLOC_MEM_WORK_ITEM,
+	NDIS_FREE_MEM_WORK_ITEM, NDIS_IO_WORK_ITEM,
 	NDIS_RETURN_PACKET_WORK_ITEM,
 };
 
 struct ndis_work_entry {
 	struct list_head list;
 	enum ndis_work_entry_type type;
-	struct ndis_handle *handle;
+	struct wrapper_dev *wd;
 	union {
 		struct ndis_sched_work_item *sched_work_item;
 		struct ndis_alloc_mem_work_item alloc_mem_work_item;
@@ -418,7 +387,7 @@ struct ndis_irq {
 	void *dpc;
 
 	struct kdpc intr_dpc;
-	struct ndis_handle *handle;
+	struct wrapper_dev *wd;
 	UCHAR dpc_count;
 	/* unsigned char filler1 is used for enabled */
 	UCHAR enabled;
@@ -433,12 +402,9 @@ struct ndis_binary_data {
 };
 
 enum ndis_config_param_type {
-	NDIS_CONFIG_PARAM_INT,
-	NDIS_CONFIG_PARAM_HEXINT,
-	NDIS_CONFIG_PARAM_STRING,
-	NDIS_CONFIG_PARAM_MULTISTRING,
-	NDIS_CONFIG_PARAM_BINARY,
-	NDIS_CONFIG_PARAM_NONE,
+	NDIS_CONFIG_PARAM_INT, NDIS_CONFIG_PARAM_HEXINT,
+	NDIS_CONFIG_PARAM_STRING, NDIS_CONFIG_PARAM_MULTISTRING,
+	NDIS_CONFIG_PARAM_BINARY, NDIS_CONFIG_PARAM_NONE,
 };
 
 struct ndis_config_param {
@@ -463,51 +429,26 @@ struct ndis_bin_file {
 	void *data;
 };
 
-/*
- * There is one of these per driver. One per loaded driver exists.
- *
- */
-struct ndis_driver {
-	CSHORT type;
-	CSHORT size;
-	void *dev_object;
-	ULONG flags;
-	void *driver_start;
-	ULONG driver_size;
-	void *driver_section;
-	void *driver_extension;
-	struct ustring *driver_name;
-	void *hardware_database;
-	void *fast_io_dispatch;
-	void *driver_init;
-	void *driver_start_io;
-	void (*driver_unload)(struct ndis_driver *driver) STDCALL;
-	void *major_func[IRP_MJ_MAXIMUM_FUNCTION + 1];
+/* IDs used to store extensions in driver_object's custom extension */
+#define CE_NDIS_DRIVER_CLIENT_ID 1
+#define CE_MINIPORT_CLIENT_ID 2
 
-	/* rest is ndiswrapper specific info */
-	void *entry;
+struct ndis_driver {
 	struct list_head list;
+	struct driver_object *drv_obj;
 	char name[MAX_NDIS_SETTING_NAME_LEN];
 	char version[MAX_NDIS_SETTING_VALUE_LEN];
-
 	int bustype;
-
 	unsigned int num_pe_images;
 	struct pe_image pe_images[MAX_PE_IMAGES];
-
 	int num_bin_files;
 	struct ndis_bin_file *bin_files;
-
 	atomic_t users;
-	struct miniport_char miniport_char;
+	struct miniport_char miniport;
 };
 
-struct ndis_handle;
+struct wrapper_dev;
 
-/*
- * There is one of these per handeled device-id
- *
- */
 struct ndis_device {
 	struct list_head settings;
 	int bustype;
@@ -516,9 +457,9 @@ struct ndis_device {
 	int subvendor;
 	int subdevice;
 
-	struct ndis_driver *driver;
+	struct ndis_driver *ndis_driver;
 	char driver_name[MAX_DRIVER_NAME_LEN];
-	struct ndis_handle *handle;
+	struct wrapper_dev *wd;
 	char conf_file_name[MAX_DRIVER_NAME_LEN];
 };
 
@@ -540,7 +481,7 @@ struct ndis_wireless_stats {
 
 enum ndis_status_type {
 	Ndis802_11StatusType_Authentication,
-	Ndis802_11StatusType_PMKID_CandidateList,
+	Ndis802_11StatusType_PMKID_CandidateList = 2,
 	Ndis802_11StatusType_MediaStreamMode,
 	Ndis802_11StatusType_RadioState,
 };
@@ -551,8 +492,7 @@ struct ndis_status_indication
 };
 
 enum ndis_radio_status {
-	Ndis802_11RadioStatusOn,
-	Ndis802_11RadioStatusHardwareOff,
+	Ndis802_11RadioStatusOn, Ndis802_11RadioStatusHardwareOff,
 	Ndis802_11RadioStatusSoftwareOff,
 };
 
@@ -563,27 +503,18 @@ struct ndis_radio_status_indication
 };
 
 enum wrapper_work {
-	LINK_STATUS_CHANGED,
-	SET_INFRA_MODE,
-	SET_ESSID,
-	SET_PACKET_FILTER,
-	COLLECT_STATS,
-	SUSPEND_RESUME,
+	LINK_STATUS_CHANGED, SET_INFRA_MODE, SET_ESSID, SET_PACKET_FILTER,
+	COLLECT_STATS, SUSPEND_RESUME, HANGCHECK,
 	/* do not work when this is set */
 	SHUTDOWN
 };
 
 enum ndis_attributes {
-	ATTR_SERIALIZED,
-	ATTR_SURPRISE_REMOVE,
-	ATTR_HALT_ON_SUSPEND,
+	ATTR_SERIALIZED, ATTR_SURPRISE_REMOVE, ATTR_HALT_ON_SUSPEND,
 };
 
 enum hw_status {
-	HW_NORMAL,
-	HW_SUSPENDED,
-	HW_HALTED,
-	HW_UNAVAILABLE,
+	HW_NORMAL, HW_SUSPENDED, HW_HALTED, HW_UNAVAILABLE,
 };
 
 struct encr_info {
@@ -600,21 +531,15 @@ struct ndis_essid {
 };
 
 enum network_infrastructure {
-	Ndis802_11IBSS,
-	Ndis802_11Infrastructure,
-	Ndis802_11AutoUnknown,
+	Ndis802_11IBSS, Ndis802_11Infrastructure, Ndis802_11AutoUnknown,
 	Ndis802_11InfrastructureMax
 };
 
 enum authentication_mode {
-	Ndis802_11AuthModeOpen,
-	Ndis802_11AuthModeShared,
-	Ndis802_11AuthModeAutoSwitch,
-	Ndis802_11AuthModeWPA,
-	Ndis802_11AuthModeWPAPSK,
-	Ndis802_11AuthModeWPANone,
-	Ndis802_11AuthModeWPA2,
-	Ndis802_11AuthModeWPA2PSK,
+	Ndis802_11AuthModeOpen, Ndis802_11AuthModeShared,
+	Ndis802_11AuthModeAutoSwitch, Ndis802_11AuthModeWPA,
+	Ndis802_11AuthModeWPAPSK, Ndis802_11AuthModeWPANone,
+	Ndis802_11AuthModeWPA2, Ndis802_11AuthModeWPA2PSK,
 	Ndis802_11AuthModeMax
 };
 
@@ -627,10 +552,8 @@ enum encryption_status {
 	Ndis802_11Encryption1KeyAbsent = Ndis802_11WEPKeyAbsent,
 	Ndis802_11WEPNotSupported,
 	Ndis802_11EncryptionNotSupported = Ndis802_11WEPNotSupported,
-	Ndis802_11Encryption2Enabled,
-	Ndis802_11Encryption2KeyAbsent,
-	Ndis802_11Encryption3Enabled,
-	Ndis802_11Encryption3KeyAbsent
+	Ndis802_11Encryption2Enabled, Ndis802_11Encryption2KeyAbsent,
+	Ndis802_11Encryption3Enabled, Ndis802_11Encryption3KeyAbsent
 };
 
 struct ndis_timer {
@@ -643,7 +566,7 @@ struct ndis_miniport_timer {
 	struct kdpc kdpc;
 	void *timer_func;
 	void *timer_ctx;
-	struct ndis_handle *handle;
+	struct wrapper_dev *wd;
 	struct ndis_miniport_timer *next;
 };
 
@@ -699,6 +622,9 @@ struct ndis_resource_list {
 	struct ndis_resource_entry list[0];
 };
 
+/* number of resources to be allocated by NdisMPciAssignResources */
+#define MAX_NDIS_PCI_RESOURCES 20
+
 struct ndis_event {
 	struct kevent kevent;
 };
@@ -714,6 +640,8 @@ struct ndis_reference {
 	BOOLEAN closing;
 };
 
+struct ndis_miniport_block;
+
 struct ndis_miniport_interrupt {
 	void *object;
 	KSPIN_LOCK dpc_count_lock;
@@ -740,29 +668,20 @@ struct ndis_filterdbs {
 };
 
 enum ndis_interface_type {
-	NdisInterfaceInternal,
-	NdisInterfaceIsa,
-	NdisInterfaceEisa,
-	NdisInterfaceMca,
-	NdisInterfaceTurboChannel,
-	NdisInterfacePci,
+	NdisInterfaceInternal, NdisInterfaceIsa, NdisInterfaceEisa,
+	NdisInterfaceMca, NdisInterfaceTurboChannel, NdisInterfacePci,
 	NdisInterfacePcMcia,
 };
 
 /*
- * This is the per device struct. One per PCI-device exists.
- *
  * This struct contains function pointers that the drivers references
  * directly via macros, so it's important that they are at the correct
- * position hence the paddings.
+ * position.
  */
-/* This is called NDIS_MINIPORT_BLOCK in NDIS; however, when using it
- * in Ndis functions, it is passed as void *handle. To avoid
- * unnecessary typecasting, we declare it as ndis_handle */
-struct ndis_handle {
+struct ndis_miniport_block {
 	void *signature;
-	struct ndis_handle *next;
-	struct ndis_driver *driver;
+	struct ndis_miniport_block *next;
+	struct driver_object *drv_obj;
 	void *adapter_ctx;
 	struct unicode_string name;
 	struct ndis_bind_paths *bindpaths;
@@ -773,17 +692,17 @@ struct ndis_handle {
 	UCHAR lock_acquired;
 	UCHAR pmode_opens;
 	UCHAR assigned_cpu;
-	KSPIN_LOCK kspin_lock;
+	KSPIN_LOCK lock;
 	enum ndis_request_type *mediarequest;
 	struct ndis_miniport_interrupt *interrupt;
 	ULONG flags;
 	ULONG pnp_flags;
-	struct list_entry packet_list;
+	struct nt_list packet_list;
 	struct ndis_packet *first_pending_tx_packet;
 	struct ndis_packet *return_packet_queue;
 	ULONG request_buffer;
 	void *set_mcast_buffer;
-	struct ndis_handle *primary_miniport;
+	struct ndis_miniport_block *primary_miniport;
 	void *wrapper_ctx;
 	void *bus_data_ctx;
 	ULONG pnp_capa;
@@ -806,9 +725,9 @@ struct ndis_handle {
 	ULONG bus_number;
 	enum ndis_interface_type bus_type;
 	enum ndis_interface_type adapter_type;
-	struct device_object *device_obj;
-	struct device_object *phys_device_obj;
-	struct device_object *next_device_obj;
+	struct device_object *fdo;
+	struct device_object *pdo;
+	struct device_object *next_device;
 	void *mapreg;
 	void *call_mgraflist;
 	void *miniport_thread;
@@ -847,17 +766,14 @@ struct ndis_handle {
 	void *wan_tx_complete;
 	void *wan_rx;
 	void *wan_rx_complete;
+	/* ndiswrapper specific */
+	struct wrapper_dev *wd;
+};
 
-	/* the rest are ndiswrapper specific */
-
-	/* keep a barrier in cases of over-stepping */
-	char barrier[200];
-
-	union {
-		struct pci_dev *pci;
-		struct usb_device *usb;
-		void *ptr;
-	} dev;
+struct wrapper_dev {
+	struct ndis_miniport_block *nmb;
+	struct ndis_driver *driver;
+	struct phys_dev dev;
 #if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,0)
 	struct usb_interface *intf;
 #endif
@@ -865,7 +781,6 @@ struct ndis_handle {
 	void *shutdown_ctx;
 
 	struct work_struct irq_work;
-
 	struct ndis_irq *ndis_irq;
 	unsigned long mem_start;
 	unsigned long mem_end;
@@ -873,7 +788,7 @@ struct ndis_handle {
 	struct net_device_stats stats;
 	struct iw_statistics wireless_stats;
 	struct ndis_wireless_stats ndis_stats;
-	struct ndis_device *device;
+	struct ndis_device *ndis_device;
 
 	struct work_struct xmit_work;
 	struct ndis_packet *xmit_ring[XMIT_RING_SIZE];
@@ -923,9 +838,9 @@ struct ndis_handle {
 
 	mac_address mac;
 
-	/* List of initialized timers */
-	struct list_head timers;
-	KSPIN_LOCK timers_lock;
+	/* list of initialized timers */
+	struct nt_list wrapper_timer_list;
+	KSPIN_LOCK timer_lock;
 
 	struct proc_dir_entry *procfs_iface;
 
@@ -935,25 +850,44 @@ struct ndis_handle {
 	unsigned long attributes;
 	struct ndis_resource_list *pci_resources;
 
+	int iw_auth_set;
+	int iw_auth_wpa_version;
+	int iw_auth_cipher_pairwise;
+	int iw_auth_cipher_group;
+	int iw_auth_key_mgmt;
+	int iw_auth_80211_auth_alg;
+};
+
+struct ndis_pmkid_candidate
+{
+	mac_address bssid;
+	unsigned long flags;
+};
+
+struct ndis_pmkid_candidate_list
+{
+	unsigned long version;
+	unsigned long num_candidates;
+	struct ndis_pmkid_candidate candidates[1];
 };
 
 struct ndis_packet *allocate_ndis_packet(void);
 void free_ndis_packet(struct ndis_packet *packet);
-STDCALL void NdisMIndicateReceivePacket(struct ndis_handle *handle,
+STDCALL void NdisMIndicateReceivePacket(struct ndis_miniport_block *nmb,
 					struct ndis_packet **packets,
 					UINT nr_packets);
-STDCALL void NdisMSendComplete(struct ndis_handle *handle,
+STDCALL void NdisMSendComplete(struct ndis_miniport_block *nmb,
 			       struct ndis_packet *packet, NDIS_STATUS status);
-STDCALL void NdisMSendResourcesAvailable(struct ndis_handle *handle);
-STDCALL void NdisMIndicateStatus(struct ndis_handle *handle,
+STDCALL void NdisMSendResourcesAvailable(struct ndis_miniport_block *nmb);
+STDCALL void NdisMIndicateStatus(struct ndis_miniport_block *nmb,
 				 NDIS_STATUS status, void *buf, UINT len);
-STDCALL void NdisMIndicateStatusComplete(struct ndis_handle *handle);
-STDCALL void NdisMQueryInformationComplete(struct ndis_handle *handle,
+STDCALL void NdisMIndicateStatusComplete(struct ndis_miniport_block *nmb);
+STDCALL void NdisMQueryInformationComplete(struct ndis_miniport_block *nmb,
 					   NDIS_STATUS status);
-STDCALL void NdisMSetInformationComplete(struct ndis_handle *handle,
+STDCALL void NdisMSetInformationComplete(struct ndis_miniport_block *nmb,
 					 NDIS_STATUS status);
-STDCALL void NdisMResetComplete(struct ndis_handle *handle, NDIS_STATUS status,
-				BOOLEAN address_reset);
+STDCALL void NdisMResetComplete(struct ndis_miniport_block *nmb,
+				NDIS_STATUS status, BOOLEAN address_reset);
 STDCALL ULONG NDIS_BUFFER_TO_SPAN_PAGES(ndis_buffer *buffer);
 STDCALL BOOLEAN NdisWaitEvent(struct ndis_event *event, UINT timeout);
 STDCALL void NdisSetEvent(struct ndis_event *event);
@@ -962,18 +896,18 @@ STDCALL void EthRxIndicateHandler(void *adapter_ctx, void *rx_ctx,
 				  char *header1, char *header,
 				  UINT header_size, void *look_ahead,
 				  UINT look_ahead_size, UINT packet_size);
-STDCALL void EthRxComplete(struct ndis_handle *handle);
-STDCALL void NdisMTransferDataComplete(struct ndis_handle *handle,
+STDCALL void EthRxComplete(struct ndis_miniport_block *nmb);
+STDCALL void NdisMTransferDataComplete(struct ndis_miniport_block *nmb,
 				       struct ndis_packet *packet,
 				       NDIS_STATUS status, UINT bytes_txed);
 STDCALL void NdisWriteConfiguration(NDIS_STATUS *status,
-				    struct ndis_handle *handle,
+				    struct ndis_miniport_block *nmb,
 				    struct unicode_string *key,
 				    struct ndis_config_param *val);
 
 void *get_sp(void);
 int ndis_init(void);
-void ndis_exit_handle(struct ndis_handle *handle);
+void ndis_exit_device(struct wrapper_dev *wd);
 void ndis_exit(void);
 
 int usb_init(void);
@@ -983,20 +917,13 @@ void usb_cleanup(void);
 int load_pe_images(struct pe_image[], int n);
 
 int ndiswrapper_procfs_init(void);
-int ndiswrapper_procfs_add_iface(struct ndis_handle *handle);
-void ndiswrapper_procfs_remove_iface(struct ndis_handle *handle);
+int ndiswrapper_procfs_add_iface(struct wrapper_dev *wd);
+void ndiswrapper_procfs_remove_iface(struct wrapper_dev *wd);
 void ndiswrapper_procfs_remove(void);
 
 int misc_funcs_init(void);
-void misc_funcs_exit_handle(struct ndis_handle *handle);
+void misc_funcs_exit_device(struct wrapper_dev *wd);
 void misc_funcs_exit(void);
-void *wrap_kmalloc(size_t size, int flags);
-void wrap_kfree(void *ptr);
-void wrapper_init_timer(struct ktimer *ktimer, void *handle);
-int wrapper_set_timer(struct wrapper_timer *wrapper_timer,
-                      unsigned long expires, unsigned long repeat,
-                      struct kdpc *kdpc);
-void wrapper_cancel_timer(struct wrapper_timer *wrapper_timer, char *canceled);
 
 void packet_recycler(void *param);
 int stricmp(const char *s1, const char *s2);
