@@ -30,6 +30,7 @@
 #include <linux/random.h>
 #include <linux/ctype.h>
 #include <linux/usb.h>
+#include <linux/list.h>
 
 #include <linux/spinlock.h>
 #include <asm/mman.h>
@@ -48,7 +49,8 @@
 #define CONFIG_USB 1
 #endif
 
-#define addr_offset(driver) (__builtin_return_address(0) - (driver)->entry)
+#define addr_offset(drvr) (__builtin_return_address(0) - \
+			     (drvr)->drv_obj->driver_start)
 
 /* Workqueue / task queue backwards compatibility stuff */
 #if LINUX_VERSION_CODE > KERNEL_VERSION(2,5,41)
@@ -61,7 +63,7 @@ typedef struct workqueue_struct *workqueue;
 
 #define PCI_DMA_ALLOC_COHERENT(pci_dev,size,dma_handle) \
 	dma_alloc_coherent(&pci_dev->dev,size,dma_handle, \
-			   GFP_KERNEL | __GFP_REPEAT)
+			   GFP_KERNEL | __GFP_REPEAT | GFP_DMA)
 #define PCI_DMA_FREE_COHERENT(pci_dev,size,cpu_addr,dma_handle) \
 	dma_free_coherent(&pci_dev->dev,size,cpu_addr,dma_handle)
 #define PCI_DMA_MAP_SINGLE(pci_dev,addr,size,direction) \
@@ -168,7 +170,7 @@ typedef task_queue workqueue;
 
 #endif // LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,0)
 
-#ifdef CONFIG_SOFTWARE_SUSPEND2
+#if defined(CONFIG_SOFTWARE_SUSPEND2) || defined(CONFIG_SUSPEND2)
 #define KTHREAD_RUN(a,b,c) kthread_run(a,b,0,c)
 #else
 #define KTHREAD_RUN(a,b,c) kthread_run(a,b,c)
@@ -298,16 +300,16 @@ do {									\
 #define KMALLOC_THRESHOLD 131072
 
 /* TICK is 100ns */
-#define TICKSPERSEC             10000000
-#define SECSPERDAY              86400
+#define TICKSPERSEC		10000000
+#define TICKSPERMSEC		10000
+#define SECSPERDAY		86400
+#define SECSPERHOUR		3600
+#define SECSPERMIN		60
+#define DAYSPERWEEK		7
 
 /* 1601 to 1970 is 369 years plus 89 leap days */
-#define SECS_1601_TO_1970       ((369 * 365 + 89) * (u64)SECSPERDAY)
-#define TICKS_1601_TO_1970      (SECS_1601_TO_1970 * TICKSPERSEC)
-
-#define UNIMPL() do {							\
-		printk(KERN_ERR "%s --UNIMPLEMENTED--\n", __FUNCTION__ ); \
-	} while (0)
+#define SECS_1601_TO_1970	((369 * 365 + 89) * (u64)SECSPERDAY)
+#define TICKS_1601_TO_1970	(SECS_1601_TO_1970 * TICKSPERSEC)
 
 typedef void (*WRAP_EXPORT_FUNC)(void);
 
@@ -320,10 +322,12 @@ struct wrap_export {
 #define WRAP_EXPORT_SYMBOL(f) {#f, (WRAP_EXPORT_FUNC)x86_64_ ## f}
 #define WRAP_EXPORT_WIN_FUNC(f) {#f, (WRAP_EXPORT_FUNC)x86_64__win_ ## f}
 #define WRAP_FUNC_PTR(f) &x86_64_ ## f
+#define WRAP_FUNC_PTR_DECL(f) void x86_64_ ## f(void);
 #else
 #define WRAP_EXPORT_SYMBOL(f) {#f, (WRAP_EXPORT_FUNC)f}
 #define WRAP_EXPORT_WIN_FUNC(f) {#f, (WRAP_EXPORT_FUNC)_win_ ## f}
 #define WRAP_FUNC_PTR(f) &f
+#define WRAP_FUNC_PTR_DECL(f)
 #endif
 /* map name s to function f - if f is different from s */
 #define WRAP_EXPORT_MAP(s,f)
@@ -352,22 +356,36 @@ extern KSPIN_LOCK cancel_lock;
 
 #define WRAPPER_TIMER_MAGIC 47697249
 struct wrapper_timer {
-	struct list_head list;
+	/* wrapper_timer replaces kdpc field in ktimer; if some
+	 * (nasty) driver passes ktimer->kdpc to other functions,
+	 * having kdpc as first field of wrapper_timer will work as
+	 * expected */
+	struct kdpc *kdpc;
+	struct nt_list list;
 	struct timer_list timer;
+	struct ktimer *ktimer;
 #ifdef DEBUG_TIMER
 	unsigned long wrapper_timer_magic;
 #endif
 	long repeat;
 	int active;
-	struct ktimer *ktimer;
-	struct kdpc *kdpc;
 	KSPIN_LOCK lock;
 };
 
 typedef struct mdl ndis_buffer;
 
+struct phys_dev {
+	int dev_type;
+	struct pci_dev *pci;
+	struct usb_device *usb;
+};
+
 int ntoskernel_init(void);
 void ntoskernel_exit(void);
+struct driver_object *find_bus_driver(const char *name);
+struct device_object *alloc_pdo(struct driver_object *drv_obj,
+				struct phys_dev *dev);
+
 STDCALL void *ExAllocatePoolWithTag(enum pool_type pool_type, SIZE_T size,
 				    ULONG tag);
 STDCALL void ExFreePool(void *p);
@@ -376,16 +394,56 @@ STDCALL void KeInitializeEvent(struct kevent *kevent,
 			       enum event_type type, BOOLEAN state);
 STDCALL LONG KeSetEvent(struct kevent *kevent, KPRIORITY incr, BOOLEAN wait);
 STDCALL LONG KeResetEvent(struct kevent *kevent);
-STDCALL NTSTATUS KeWaitForSingleObject(void *object, KWAIT_REASON reason,
-					KPROCESSOR_MODE waitmode,
-					BOOLEAN alertable,
-					LARGE_INTEGER *timeout);
+STDCALL void KeClearEvent(struct kevent *kevent);
+STDCALL void KeInitializeDpc(struct kdpc *kdpc, void *func, void *ctx);
+BOOLEAN insert_kdpc_work(struct kdpc *kdpc);
+BOOLEAN remove_kdpc_work(struct kdpc *kdpc);
+STDCALL BOOLEAN KeInsertQueueDpc(struct kdpc *kdpc, void *arg1, void *arg2);
+STDCALL BOOLEAN KeRemoveQueueDpc(struct kdpc *kdpc);
+STDCALL NTSTATUS KeWaitForSingleObject(struct kevent *object,
+				       KWAIT_REASON reason,
+				       KPROCESSOR_MODE waitmode,
+				       BOOLEAN alertable,
+				       LARGE_INTEGER *timeout);
 struct mdl *allocate_init_mdl(void *virt, ULONG length);
 void free_mdl(struct mdl *mdl);
 STDCALL struct mdl *IoAllocateMdl(void *virt, ULONG length, BOOLEAN second_buf,
 				  BOOLEAN charge_quota, struct irp *irp);
 STDCALL void IoFreeMdl(struct mdl *mdl);
 STDCALL void NdisFreeBuffer(ndis_buffer *buffer);
+_FASTCALL LONG InterlockedDecrement(FASTCALL_DECL_1(LONG volatile *val));
+_FASTCALL LONG InterlockedIncrement(FASTCALL_DECL_1(LONG volatile *val));
+_FASTCALL struct nt_list *
+ExInterlockedInsertHeadList(FASTCALL_DECL_3(struct nt_list *head,
+					    struct nt_list *entry,
+					    KSPIN_LOCK *lock));
+_FASTCALL struct nt_list *
+ExInterlockedInsertTailList(FASTCALL_DECL_3(struct nt_list *head,
+					    struct nt_list *entry,
+					    KSPIN_LOCK *lock));
+_FASTCALL struct nt_list *
+ExInterlockedRemoveHeadList(FASTCALL_DECL_2(struct nt_list *head,
+					    KSPIN_LOCK *lock));
+STDCALL NTSTATUS IoCreateDevice(struct driver_object *driver,
+				ULONG dev_ext_length,
+				struct unicode_string *dev_name,
+				DEVICE_TYPE dev_type,
+				ULONG dev_chars, BOOLEAN exclusive,
+				struct device_object **dev_obj);
+STDCALL void IoDeleteDevice(struct device_object *dev);
+STDCALL void IoDetachDevice(struct device_object *topdev);
+STDCALL NTSTATUS
+IoAllocateDriverObjectExtension(struct driver_object *drv_obj,
+				void *client_id, ULONG extlen, void **ext);
+STDCALL void *IoGetDriverObjectExtension(struct driver_object *drv,
+					 void *client_id);
+STDCALL struct device_object *IoAttachDeviceToDeviceStack
+	(struct device_object *src, struct device_object *dst);
+STDCALL void KeInitializeEvent(struct kevent *kevent, enum event_type type,
+			       BOOLEAN state);
+void free_custom_ext(struct driver_extension *drv_obj_ext);
+STDCALL NTSTATUS ndis_add_device(struct driver_object *drv_obj,
+				 struct device_object *pdo);
 ULONGLONG ticks_1601(void);
 
 STDCALL KIRQL KeGetCurrentIrql(void);
@@ -411,9 +469,21 @@ STDCALL NTSTATUS RtlAnsiStringToUnicodeString(struct unicode_string *dst,
 					       BOOLEAN dup);
 STDCALL void RtlInitAnsiString(struct ansi_string *dst, CHAR *src);
 STDCALL void RtlInitString(struct ansi_string *dst, CHAR *src);
+STDCALL void RtlInitUnicodeString(struct unicode_string *dest,
+				  const wchar_t *src);
 STDCALL void RtlFreeUnicodeString(struct unicode_string *string);
 STDCALL void RtlFreeAnsiString(struct ansi_string *string);
 
+void *wrap_kmalloc(size_t size, int flags);
+void wrap_kfree(void *ptr);
+void wrapper_init_timer(struct ktimer *ktimer, void *handle,
+			struct kdpc *kdpc);
+int wrapper_set_timer(struct wrapper_timer *wrapper_timer,
+		      unsigned long expires, unsigned long repeat,
+		      struct kdpc *kdpc);
+void wrapper_cancel_timer(struct wrapper_timer *wrapper_timer,
+			  BOOLEAN *canceled);
+
 unsigned long lin_to_win1(void *func, unsigned long);
 unsigned long lin_to_win2(void *func, unsigned long, unsigned long);
 unsigned long lin_to_win3(void *func, unsigned long, unsigned long,
@@ -433,6 +503,10 @@ unsigned long lin_to_win6(void *func, unsigned long, unsigned long,
 #define ERROR(fmt, ...) MSG(KERN_ERR, fmt , ## __VA_ARGS__)
 #define INFO(fmt, ...) MSG(KERN_INFO, fmt , ## __VA_ARGS__)
 
+#define UNIMPL() ERROR("--UNIMPLEMENTED--")
+
+void adjust_user_shared_data_addr(char *driver, unsigned long length);
+
 static inline KIRQL current_irql(void)
 {
 	if (in_atomic() || irqs_disabled())
@@ -444,7 +518,7 @@ static inline KIRQL current_irql(void)
 static inline KIRQL raise_irql(KIRQL newirql)
 {
 	KIRQL irql = current_irql();
-	if (irql < DISPATCH_LEVEL && newirql == DISPATCH_LEVEL) {
+	if (irql < DISPATCH_LEVEL && newirql >= DISPATCH_LEVEL) {
 		local_bh_disable();
 		preempt_disable();
 	}
@@ -454,7 +528,7 @@ static inline KIRQL raise_irql(KIRQL newirql)
 static inline void lower_irql(KIRQL oldirql)
 {
 	KIRQL irql = current_irql();
-	if (oldirql < DISPATCH_LEVEL && irql == DISPATCH_LEVEL) {
+	if (oldirql < DISPATCH_LEVEL && irql >= DISPATCH_LEVEL) {
 		preempt_enable();
 		local_bh_enable();
 	}
@@ -497,7 +571,7 @@ do {							\
 	*(lock) = KSPIN_LOCK_LOCKED;			\
 	spin_unlock(&spinlock_kspin_lock);		\
 } while (0)
-		
+
 #endif // __HAVE_ARCH_CMPXCHG
 
 #define kspin_unlock(lock) xchg(lock, KSPIN_LOCK_UNLOCKED)
@@ -509,7 +583,7 @@ do {							\
 
 #endif // CONFIG_SMP
 
-/* raise IRQL to given (higher) IRQL if necessary after locking */
+/* raise IRQL to given (higher) IRQL if necessary before locking */
 #define kspin_lock_irql(lock, newirql)					\
 ({									\
 	KIRQL _cur_irql_ = current_irql();				\
@@ -558,18 +632,6 @@ do {									\
 	preempt_enable();						\
 } while (0)
 
-static inline void wrapper_set_timer_dpc(struct wrapper_timer *wrapper_timer,
-                                         struct kdpc *kdpc)
-{
-	wrapper_timer->kdpc = kdpc;
-}
-
-static inline void init_dpc(struct kdpc *kdpc, void *func, void *ctx)
-{
-	kdpc->func = func;
-	kdpc->ctx  = ctx;
-}
-
 static inline ULONG SPAN_PAGES(ULONG_PTR ptr, SIZE_T length)
 {
 	ULONG n;
@@ -588,6 +650,7 @@ static inline ULONG SPAN_PAGES(ULONG_PTR ptr, SIZE_T length)
 #define DBGTRACE3(fmt, ...) do { }  while (0)
 #define DBGTRACE4(fmt, ...) do { } while (0)
 #define DBGTRACE5(fmt, ...) do { } while (0)
+#define DBGTRACE6(fmt, ...) do { } while (0)
 
 /* for a block of code */
 #define DBG_BLOCK() while (0)
@@ -631,12 +694,18 @@ extern int debug;
 #define DBGTRACE5(fmt, ...) DBGTRACE(5, fmt , ## __VA_ARGS__)
 #endif
 
+#if defined DEBUG && DEBUG >= 6
+#undef DBGTRACE6
+#define DBGTRACE6(fmt, ...) DBGTRACE(6, fmt , ## __VA_ARGS__)
+#endif
+
 #define TRACEENTER(fmt, ...) DBGTRACE("Enter " fmt , ## __VA_ARGS__)
 #define TRACEENTER1(fmt, ...) DBGTRACE1("Enter " fmt , ## __VA_ARGS__)
 #define TRACEENTER2(fmt, ...) DBGTRACE2("Enter " fmt , ## __VA_ARGS__)
 #define TRACEENTER3(fmt, ...) DBGTRACE3("Enter " fmt , ## __VA_ARGS__)
 #define TRACEENTER4(fmt, ...) DBGTRACE4("Enter " fmt , ## __VA_ARGS__)
 #define TRACEENTER5(fmt, ...) DBGTRACE5("Enter " fmt , ## __VA_ARGS__)
+#define TRACEENTER6(fmt, ...) DBGTRACE6("Enter " fmt , ## __VA_ARGS__)
 
 #define TRACEEXIT(stmt) do { DBGTRACE("Exit"); stmt; } while(0)
 #define TRACEEXIT1(stmt) do { DBGTRACE1("Exit"); stmt; } while(0)
@@ -644,8 +713,9 @@ extern int debug;
 #define TRACEEXIT3(stmt) do { DBGTRACE3("Exit"); stmt; } while(0)
 #define TRACEEXIT4(stmt) do { DBGTRACE4("Exit"); stmt; } while(0)
 #define TRACEEXIT5(stmt) do { DBGTRACE5("Exit"); stmt; } while(0)
+#define TRACEEXIT6(stmt) do { DBGTRACE6("Exit"); stmt; } while(0)
 
-#ifdef USB_DEBUG
+#if defined(DEBUG) && defined(USB_DEBUG)
 #define USBTRACE(fmt, ...) DBGTRACE1(fmt, ## __VA_ARGS__)
 #define USBTRACEENTER(fmt, ...) TRACEENTER1(fmt, ## __VA_ARGS__)
 #define USBTRACEEXIT(stmt) TRACEEXIT1(stmt)
